from django.conf import settings


def update_dns_record(dns_record, ip):
    if dns_record.provider:
        config = settings.DYNAMICDNS_PROVIDERS[dns_record.provider]
        mod_path, mod_name = config['plugin'].rsplit('.', 1)
        exec('from {} import {} as DnsPlugin'.format(mod_path, mod_name))
        dns_plugin = DnsPlugin(dns_record.domain, config)
        dns_plugin.update(ip)


import json
from time import sleep

from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt

from models import DnsRecord


def dynamic_dns_read(request, domain):
    data = {}
    dns_record = DnsRecord.objects.get(domain=domain)
    data['domain'] = dns_record.domain
    data['ip'] = dns_record.ip
    data['last_change'] = dns_record.last_change.isoformat()
    return HttpResponse(json.dumps(data), mimetype="application/json")


@csrf_exempt
def dynamic_dns_update(request, domain):
    '''
    Updates a domain to use a new IP address. If the IP should be the machine
    that performs the update, just supply your secret key as a POST parameter
    like this:
      curl http://localhost:8000/domain/update/a.example.com/ --data "key=ZHXPu3RTfs3oAexrwBTi8DGN5lmiH3t1pc9iGG1NZsp75UeM84"
    Otherwise provide an 'ip' as another parameter:
      curl http://localhost:8000/domain/update/a.example.com/ --data "key=ZHXPu3RTfs3oAexrwBTi8DGN5lmiH3t1pc9iGG1NZsp75UeM84&ip=1.2.3.4"
    You should use HTTPS in production to avoid MITM attacks.
    '''
    data = {}
    status = 400
    dns_record = None
    updated = False

    if 'key' not in request.POST:
        data['error'] = 'Bad request'
    else:
        try:
            dns_record = DnsRecord.objects.get(domain=domain)
        except DnsRecord.DoesNotExist:
            pass  # We don't want to give away which domains are managed so we will return an auth failure
        if not dns_record or request.POST['key'] != dns_record.key:
            sleep(2)  # Very basic rate-limiting (also depends on number of workers running)
            data['error'] = 'Authentication failure'
            status = 403
        else:
            if 'ip' in request.POST:
                ip = request.POST['ip']
            else:
                ip = request.META.get('HTTP_X_FORWARDED_FOR', '') or request.META.get('REMOTE_ADDR')
            if ip != dns_record.ip:
                dns_record.ip = ip
                dns_record.save()
                updated = True
            data['domain'] = dns_record.domain
            data['ip'] = dns_record.ip
            data['last_change'] = dns_record.last_change.isoformat()
            data['updated'] = updated
            status = 200
    return HttpResponse(json.dumps(data), mimetype="application/json", status=status)



from django.contrib import admin
from models import DnsRecord


class DnsRecordAdmin(admin.ModelAdmin):
    list_display = ('domain', 'ip', 'key', 'provider', 'last_change',)
    ordering = ('domain',)
    list_filter = ('provider', 'last_change')

admin.site.register(DnsRecord, DnsRecordAdmin)



from django.conf.urls import patterns, include, url


urlpatterns = patterns('dynamicdns.views',
    url(r'^update/(?P<domain>[a-z0-9-.]+)/', 'dynamic_dns_update'),
    url(r'^read/(?P<domain>[a-z0-9-.]+)/', 'dynamic_dns_read'),
)



import os
from setuptools import setup

setup(
    name = "django-dynamic-dns",
    version = "0.1.3",
    author = "Damian Moore",
    author_email = "django-dynamic-dns@epixstudios.co.uk",
    description = ("Machines on dynamic IPs can call this service at regular intervals (e.g. via cron) and when the server notices an address change it makes the relevant API call to update the DNS provider. Alternatively a simple DNS server is included which can be run instead of using an external DNS provider."),
    license = "BSD",
    keywords = "django dynamic dns server ip domain service",
    url = "https://github.com/damianmoore/django-dynamic-dns",
    download_url = 'https://github.com/damianmoore/django-dynamic-dns/archive/0.1.3.zip',
    packages = [
        'dynamicdns',
    ],
)



from datetime import datetime
import string
import random

from django.conf import settings
from django.db import models

from utils import update_dns_record


PROVIDER_CHOICES = [(name, name) for name in settings.DYNAMICDNS_PROVIDERS.keys()]


class DnsRecord(models.Model):
    domain = models.CharField(max_length=50, unique=True, help_text='Domain/subdomain name')
    ip = models.CharField(max_length=16, blank=True)
    lan_ip = models.CharField(max_length=16, blank=True, help_text='Only required when using the internal DNS server as a provider')
    key = models.CharField(max_length=50, blank=True, help_text='Optional - Autogenerated if left blank')
    provider = models.CharField(max_length=25, choices=PROVIDER_CHOICES, blank=True)
    last_change = models.DateTimeField(blank=True, null=True)

    def __unicode__(self):
        return self.domain

    def generate_key(self, size=50, chars=string.ascii_uppercase + string.ascii_lowercase + string.digits):
        return ''.join(random.choice(chars) for x in range(size))

    def save(self):
        if not self.key:
            self.key = self.generate_key()
        update_dns_record(self, self.ip)
        self.last_change = datetime.now()
        super(DnsRecord, self).save()
